// Sample05.cpp : Defines the entry point for the console application.
//

#include "stdafx.h"

ULONGLONG g_StartTime;
ULONG g_TxPackets;
ULONGLONG g_TxBytes;

#define MAX_PACKETS		500000

VOID ProcessPackets(TC_PACKETS_BUFFER buffer);

#ifdef _WIN32
static __inline ULONGLONG get_milliseconds()
{

	clock_t time;
	ULONGLONG result;

	time = clock();

	result = (ULONGLONG)time * 1000 / CLK_TCK;

	return result;
}
#else
static inline ULONGLONG get_milliseconds()
{
	struct timeval tv;
	ULONGLONG result;

	gettimeofday(&tv, NULL);
	result = (ULONGLONG)tv.tv_sec * 1000000 + tv.tv_usec;
	
	result /= 1000;

	return result;
}
#endif




int main(int argc, char* argv[])
{
	PTC_PORT	portList;
	ULONG		numPorts;
	TC_STATUS	status;
	ULONG		i;
	ULONG		numPort;
	TC_PORT	port;
	TC_INSTANCE instance;

	UNUSED(argc);
	UNUSED(argv);

	status = TcQueryPortList(&portList, &numPorts);

	if (status != TC_SUCCESS)
	{
		printf("Cannot retrieve port list, %s\n", TcStatusGetString(status));
		return -1;
	}

	if (numPorts == 0)
	{
		printf("No ports available.\n");
		return 0;
	}
	
	do
	{
		for (i = 0; i < numPorts; i++)
		{
			printf("%u) %s (%s)\n", i, TcPortGetDescription(portList[i]), TcPortGetName(portList[i]));
		}

		printf("\nPort number? ");
	}
	while((scanf_s("%u", &numPort) == 0 ) || (numPort >= numPorts) );

	port = portList[numPort];

	status = TcInstanceOpenByPort(port, &instance);

	TcFreePortList(portList);

	if (status != TC_SUCCESS)
	{
		printf("Cannot open port, %08x\n", status);
		return -1;
	}

	//
	// Enable reception
	//
	status = TcInstanceSetFeature(instance, TC_INST_FT_RX_STATUS, 1);
	if (status != TC_SUCCESS)
	{
		printf("Error, cannot enable reception: %s (%08x)", TcStatusGetString(status), status);
		
		(VOID)TcInstanceClose(instance);
        return -1;
	}

	do
	{
		TC_PACKETS_BUFFER hPacketsBuffer;
		
		status = TcInstanceReceivePackets(instance, &hPacketsBuffer);
		if (status != TC_SUCCESS) break;
		

		//
		// check if we have received any packet
		//
		if (hPacketsBuffer == NULL) continue;

		ProcessPackets(hPacketsBuffer);

		//
		// the packets in the buffer, being them returned by TcPacketsBufferGetNextPacket
		// or thru the raw pointers, are valid until the user calls PacketsBufferDestroy
		// or closes the Adapter
		//
		TcPacketsBufferDestroy(hPacketsBuffer);

	}while(!_kbhit());

	TcInstanceClose(instance);
	return 0;

}

VOID ProcessPackets(TC_PACKETS_BUFFER buffer)
{
	DWORD offset = 0;
	ULONGLONG stopTime;
	TC_PACKET_HEADER header;
	PBYTE packet;
	ULONG index;

	do
	{
		if (TcPacketsBufferQueryNextPacket(buffer, &header, (PVOID*)&packet) != TC_SUCCESS)
			break;

		index = *(PULONG)packet;
		
		offset += header.Length;

		g_TxPackets ++;
		g_TxBytes += header.Length + 20;

		if (g_TxPackets > MAX_PACKETS)
		{
			float elapsedTime;
			float bandwidth;

			/* take the time to compute the bandwidth */
			stopTime = get_milliseconds();

			/* compute the elapsed time since the last statistics printout */
			elapsedTime = (float)(LONGLONG)(stopTime - g_StartTime) / 1000;

			/* compute an average bandwidth, 
			* valid on ethernet with frames containing the fcs 
			*/
			bandwidth = (float)(g_TxBytes * 8) / elapsedTime;

			printf("Received %d packets in %f seconds\t Bandwidth = %.2f Mbps\n", 
				g_TxPackets,
				elapsedTime,
				bandwidth / (1000 * 1000)
				);

			/* reset the counters */
			g_TxPackets = 0;
			g_TxBytes = 0;
			g_StartTime = stopTime;
		}
	}while(TRUE);
}
