// Sample08.cpp : Defines the entry point for the console application.
//

#include "stdafx.h"

#define BUFFER_SIZE		(256 * 1024)

TC_STATUS TransmitPackets(TC_INSTANCE instance, USHORT packetSize);

#ifdef _WIN32
static __inline ULONGLONG get_milliseconds()
{
	clock_t time;
	ULONGLONG result;

	time = clock();

	result = (ULONGLONG)time * 1000 / CLK_TCK;

	return result;
}
#else
static inline ULONGLONG get_milliseconds()
{
	struct timeval tv;
	ULONGLONG result;

	gettimeofday(&tv, NULL);
	result = (ULONGLONG)tv.tv_sec * 1000000 + tv.tv_usec;
	
	result /= 1000;

	return result;
}
#endif

int main(int argc, char* argv[])
{
	PTC_PORT	portList;
	ULONG		numPorts;
	TC_STATUS	status;
	ULONG		i;
	ULONG		numPort;
	ULONG		value;
	USHORT		packetSize;
	TC_PORT	port;
	TC_INSTANCE instance;

	UNUSED(argc);
	UNUSED(argv);

	status = TcQueryPortList(&portList, &numPorts);

	if (status != TC_SUCCESS)
	{
		printf("Cannot retrieve port list, %s\n", TcStatusGetString(status));
		return -1;
	}

	if (numPorts == 0)
	{
		printf("No ports available.\n");
		return 0;
	}
	
	do
	{
		for (i = 0; i < numPorts; i++)
		{
			if (TcPortGetType(portList[i]) == TC_PORT_TYPE_PHYSICAL)
			{
				printf("%u) %s (%s)\n", i, TcPortGetDescription(portList[i]), TcPortGetName(portList[i]));
			}
		}

		printf("\nPort number? ");
	}
	while((scanf_s("%u", &numPort) == 0 ) || (numPort >= numPorts) || (TcPortGetType(portList[numPort]) != TC_PORT_TYPE_PHYSICAL));

	port = portList[numPort];

	status = TcInstanceOpenByPort(port, &instance);

	TcFreePortList(portList);

	if (status != TC_SUCCESS)
	{
		printf("Cannot open port, %08x\n", status);
		return -1;
	}

	status = TcInstanceSetFeature(instance, TC_INST_FT_TX_STATUS, 1);

	if (status != TC_SUCCESS)
	{
		printf("Cannot enable transmission, %08x\n", status);
		TcInstanceClose(instance);
		return -1;
	}

	do
	{
		printf("--- Hardware will append the correct FCS at the end of the packet ---\n");
		printf("PacketSize (excluding FCS) [60-1514]: ");

	}while(scanf_s("%u", &value) < 1 || value <60 || value > 1514);

	packetSize = (USHORT)value;

	status = TransmitPackets(instance, packetSize);

	TcInstanceClose(instance);
	return 0;

}

TC_STATUS TransmitPackets(TC_INSTANCE instance, USHORT packetSize)
{
	TC_PACKETS_BUFFER buffer;
	TC_PACKET_HEADER header;
	TC_STATUS status = TC_SUCCESS;
	DWORD maxPackets;

	BYTE packet[1514];

	ULONGLONG totalPackets = 0;
	ULONGLONG txPackets = 0;
	ULONGLONG start_time, stop_time;
	ULONG packetsInBuffer;

	//
	// 20 + 4 is the minimum interframe gap and 4 bytes FCS
	//
	maxPackets = 15 * (125 * 1000 * 1000) / (packetSize + 20 + 4);

	do
	{
		status = TcPacketsBufferCreate(BUFFER_SIZE, &buffer);
		if (status != TC_SUCCESS)
		{
			printf("Cannot create packets buffer %08x\n", status);			
			break;
		}

#ifdef _WIN32
		ZeroMemory(&header, sizeof(header));
#else
		memset(&header, 0, sizeof(header));
#endif
		header.Length = packetSize;
		header.CapturedLength = packetSize;
			
		packet[0] = 0x00;
		packet[1] = 0x11;
		packet[2] = 0x22;
		packet[3] = 0x33;
		packet[4] = 0x44;
		packet[5] = 0x01;
		packet[6] = 0x00;
		packet[7] = 0x99;
		packet[8] = 0x88;
		packet[9] = 0x77;
		packet[10] = 0x66;
		packet[11] = 0x01;
		packet[12] = 0xCA;
		packet[13] = 0xCE;

		start_time = get_milliseconds();


		do
		{
			(VOID)TcPacketsBufferSetEffectiveLength(buffer, 0);

			//
			// fill the buffer
			//
			packetsInBuffer = 0;
			do
			{
				*(PULONGLONG)(&packet[14]) = totalPackets + packetsInBuffer;
				if (TcPacketsBufferCommitNextPacket(buffer, &header, packet) != TC_SUCCESS)
				{
					break;
				}

				packetsInBuffer++;
			}while(TRUE);

			//
			// send the packets
			//
			status = TcInstanceTransmitPackets(instance, buffer);

			if (status != TC_SUCCESS)
			{
				printf("Cannot transmit, %08x\n", status);
				break;
			}

			totalPackets += packetsInBuffer;
			txPackets += packetsInBuffer;

			if (txPackets > maxPackets)
			{
				float elapsed_time;
				float bandwidth;

				//
				// take the time to compute the bandwidth
				//
				stop_time = get_milliseconds();

				//
				// compute the elapsed time since the last statistics printout
				//
				elapsed_time = (float)(LONGLONG)(stop_time - start_time) / 1000;

				//
				// compute an average bandwidth, 
				// valid on ethernet with frames containing the fcs 
				//
				// 20 + 4 is the minimum interframe gap and 4 bytes FCS
				//
				bandwidth = (float)((LONGLONG)txPackets * (packetSize + 20 + 4) * 8) / elapsed_time;

#ifdef _WIN32
				printf("Sent %I64u packets in %f seconds\t Bandwidth = %.2f Mbps\n", 
					txPackets,
					elapsed_time,
					bandwidth / (1000 * 1000)
					);
#else
				printf("Sent %llu packets in %f seconds\t Bandwidth = %.2f Mbps\n", 
					txPackets,
					elapsed_time,
					bandwidth / (1000 * 1000)
					);
#endif

				//
				// reset the counters
				//
				txPackets = 0;
				start_time = stop_time;
			}
		}while(!_kbhit());


	}while(FALSE);

	return status;
}
